# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2013, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

require 'thor'
require 'irb'

module Tomte
  module Cli

    begin
      require 'pry'
    rescue LoadError
      raise "The Pry gem is required."
    end

    autoload :Misc,     "tomte/cli/misc"
    autoload :Console,  "tomte/cli/console"

    class Base < Thor

      include Thor::Actions
      include Tomte::Cli::Misc
      include Tomte::Cli::Console

      private

      def try_require(requirement = "tomte")
        requirement = requirement.to_s
        begin
          require requirement
        rescue LoadError
          path = File.expand_path('../../lib', __FILE__)
          $:.unshift(path)
          require requirement
        end
      end

      def prepare(task)

        Tomte.configure!(:core => {:application => "tomte-console"})

        if options['log-level']
          puts "-  set log level to '#{options['log-level']}'."
          Tomte::Config.core[:log_level] = options['log-level']
        end

        if options.environment
          puts "-  set environment to '#{options.environment}'."
          Tomte::Config.core[:environment] = options.environment
        end

        if options.debug
          puts "-  set debugging on.".important!
          Tomte::Config.core[:debug] = true
          Tomte::Config.core[:log_level] = :debug
        end

        if options.help?
          help(task.to_s)
          raise SystemExit
        end

      end

      def tomte_console

        begin

          # configure Pry to use awesome_print
          Pry.config.print = proc { |output, value| output.puts value.ai }
          Pry
        rescue LoadError
          raise "The Pry gem is required."
        end
      end

      protected
      def self.banner(task=nil, *args)
        "Tomte #{task.name}"
      end

    end
  end
end
