# encoding: UTF-8
#
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2013, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte
  module Document
    extend ActiveSupport::Concern
    extend Plugins

    autoload :Keys,               'tomte/document/keys'
    autoload :Doctype,            'tomte/document/doctype'
    autoload :Clone,              'tomte/document/clone'
    autoload :Equality,           'tomte/document/equality'
    autoload :Inspect,            'tomte/document/inspect'
    autoload :Accessible,         'tomte/document/accessible'
    autoload :Log,                'tomte/document/log'
    autoload :BSONSerialize,      'tomte/document/bson_serialize'
    autoload :ToHuman,            'tomte/document/to_human'
    autoload :Serialization,      'tomte/document/serialization'

    # optional properties used in some descendants
    autoload :Enumerable,         'tomte/document/enumerable'
    autoload :Extensibility,      'tomte/document/extensibility'

    # document classes
    autoload :Extensible,         'tomte/document/extensible'

    autoload :Parser,             'tomte/document/parser'

    Dir[File.join(File.dirname(__FILE__), 'document', 'extensions', '*.rb')].each do |extension|
      require extension
    end

    include Keys
    include Doctype
    include Clone
    include Equality
    include Inspect
    include Accessible
    include Log
    include BSONSerialize
    include ToHuman
    include Serialization

    included do
      extend Plugins
    end
  end
end
