# encoding: UTF-8

# this comes (almost) verbatim from MongoMapper
module Tomte
  module Document
    module Keys
      class Key
        attr_accessor :name, :type, :options, :default_value

        def initialize(*args)
          options = args.extract_options!
          @name, @type = args.shift.to_s, args.shift
          self.options = (options || {}).symbolize_keys
          self.default_value = self.options[:default]
        end

        def ==(other)
          @name == other.name && @type == other.type
        end

        def embeddable?
          return false unless type.respond_to?(:embeddable?)
          type.embeddable?
        end

        def can_default_id?
          type && (type == Tomte::ObjectId || type == BSON::ObjectId || type == String)
        end

        def number?
          type == Integer || type == Float
        end

        def get(value)
          if value.nil? && !default_value.nil?
            if default_value.respond_to?(:call)
              return default_value.call
            else
              # Using Marshal is easiest way to get a copy of mutable objects
              # without getting an error on immutable objects
              return Marshal.load(Marshal.dump(default_value))
            end
          end

          if options[:typecast].present?
            type.from_tomte(value).map! { |v| typecast_class.from_tomte(v) }
          elsif value.is_a? Array
            Array.from_tomte(value)
          elsif type
            type.from_tomte(value)
          else
            value
          end
        end

        def try_cast(value)
          if options[:as_object].present? && Tomte::Document::Parser.parseable?(value)
            Tomte::Document::Parser.load(value)
          elsif options[:as_document].present? && Tomte::Document::Parser.is_document?(value)
            Tomte::Document::Parser.load_document(value)
          elsif options[:cast].present? && type && type.respond_to?(:from_tomte) && !value.is_a?(type)
            type.from_tomte(value)
          else
            value
          end
        end

        def set(value)
          if type
            type.to_tomte(value).tap do |values|
              if options[:typecast].present?
                values.map! { |v| typecast_class.to_tomte(v) }
              end
            end
          else
            value
          end
        end

        private
        def typecast_class
          @typecast_class ||= options[:typecast].constantize
        end
      end
    end
  end
end
