# encoding: UTF-8
require 'active_support/ordered_hash'
require 'awesome_print'

module AwesomePrint
  module Tomte
    module Document



      def self.included(base)
        base.send :alias_method, :cast_without_tomte_document, :cast
        base.send :alias_method, :cast, :cast_with_tomte_document
      end

      # Add tomte_document class names to the dispatcher pipeline.
      #------------------------------------------------------------------------------
      def cast_with_tomte_document(object, type)
        cast = cast_without_tomte_document(object, type)
        if defined?(::Tomte::Document)
          if object.is_a?(Class) && object.ancestors.include?(::Tomte::Document)
            cast = :tomte_document_class
          elsif object.class.ancestors.include?(::Tomte::OpenDocument)
            cast = :tomte_document_opendocument
          elsif object.class.ancestors.include?(::Tomte::Document)
            cast = :tomte_document_document
          #elsif object.is_a?(::Tomte::ObjectId)
          #  cast = :tomte_document_object_id
          end
        end
        cast
      end

      # Format tomte_document class object.
      #------------------------------------------------------------------------------
      def awesome_tomte_document_class(object)

        return object.inspect if !defined?(::ActiveSupport::OrderedHash) || !object.respond_to?(:keys)

        data = object.keys.sort.inject(::ActiveSupport::OrderedHash.new) do |hash, (k, v)|
          hash[k] = (v.type || "undefined").to_s.underscore.intern
          hash
        end
        "class #{object} < #{object.superclass} " << awesome_hash(data)
      end

      # Format tomte_document Document object.
      #------------------------------------------------------------------------------
      def awesome_tomte_document_document(object)
        return object.inspect if !defined?(::ActiveSupport::OrderedHash)

        data = object.key_names.sort.inject(::ActiveSupport::OrderedHash.new) do |hash, k |
          hash[k.to_sym] = object[k] unless (k.to_s.first == "_")
          hash
        end

        #data = object.attributes.sort_by {
        #    |key| key }.inject(::ActiveSupport::OrderedHash.new) do |hash, (k,v)|
        #  hash[k.to_sym] = v
        #end
        "#{object.class.name}(#{object.id}) #{awesome_hash(data)}"
      end

      # Format tomte_document Document object.
      #------------------------------------------------------------------------------
      def awesome_tomte_document_opendocument(object)
        return object.inspect if !defined?(::ActiveSupport::OrderedHash)

         data = object.table.keys.sort.inject(::ActiveSupport::OrderedHash.new) do |hash, k |
           hash[k.to_sym] = object.table[k]
           hash
         end
        "#{object.class.name}(#{object.id}) #{awesome_hash(data)}"
      end

      # Format BSON::ObjectId
      #------------------------------------------------------------------------------
      def awesome_tomte_document_object_id(object)
        object.inspect
      end
    end
  end
end

AwesomePrint::Formatter.send(:include, AwesomePrint::Tomte::Document)


module Tomte
  module Document
    module ToHuman
      extend ActiveSupport::Concern

      module ClassMethods
      end

      alias_method :to_human, :ai

    end
  end
end